/*
 * Turbo Programmer Utilities, turbo-prog-utils, www.bladox.com 
 *
 * Copyright (C) 2004 BLADOX, s.r.o.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <tfslib.h>
#include <tproglib.h>
#include <stdlib.h>
#include <string.h>

#include <stdio.h>
#include <unistd.h>

#include "../module-src/df.h"

int my_stat_size (const char *file_name);

void help ()
{
  printf ("Copy file to and from card\n");
  printf ("Usage: [args] src dst_dir\n");
  printf ("-h\thelp\n");
  printf ("-d device\tuse USB device\n");
  printf ("-sx\tsort x (0=append, 1=sms, 2=sms_skip_2, 3=sort)\n");
  printf ("-e ext\tset extension\n");
  printf ("-f\tfrom memory card\n");
  printf ("-v\tverbose\n");
  exit (-1);
}

int main (int argc, char **argv)
{
  u32 i;
  u16 c;
  u8 nb;
  u16 pos;
  u8 buf[1000];
  u8 len;
  int result;
  u8 sort;
  u16 j;
  u8 *d_name;
  u8 *s_name;
  int s_size;
  u8 *s_buf;
  MCFILE *fd;
  u8 path[1000] = { '/', '\0' };
  u8 name[1000];
  u16 name_end;
  u16 path_end;
  b32 f;
  u8 *ext = NULL;
  FILE *fs;
  u8 from = 0;
  int s_src;
  u8 *dev = USBDEVICE;

  do
  {
    result = getopt (argc, argv, "s:hfe:d:v");
    switch (result)
    {
      case 'd':
	dev = optarg;
	break;
      case 's':
	sort = atoi (optarg);
	break;
      case 'e':
	ext = optarg;
	break;
      case 'f':
	from = 1;
	break;
      case 'h':
	help ();
	break;
      case 'v':
	verbose_inc ();
	break;
    }
  }
  while (result > 0);

  if (argc - optind != 2)
    help ();

  s_src = optind;
  s_name = argv[optind++];
  d_name = argv[optind++];

  printf ("SRC %s\n", s_name);
  printf ("DST %s\n", d_name);
  if (from == 0)
  {
    s_size = my_stat_size (s_name);
    printf ("SIZE %d\n", s_size);
    s_buf = malloc (s_size);
    fs = fopen (s_name, "r");
    if (fs == NULL)
    {
      fprintf (stderr, "Cannot open %s\n", s_name);
      exit (1);
    }
    for (i = 0; i < s_size; i++)
      s_buf[i] = fgetc (fs);
    fclose (fs);

    name_end = strlen (d_name);

    for (i = name_end; i > 0; i--)
      if (d_name[i - 1] == '/')
	break;
    path_end = i;

    len = strlen (d_name);
    for (j = 0; j < name_end - path_end; j++)
      name[j] = d_name[i + j];
    name[j] = '\0';

    for (j = 0; j < path_end; j++)
      path[j] = d_name[j];

    printf ("NAME %s\n", name);
    printf ("PATH %s\n", path);

    open_device (dev);
    if (select_app (APP_DF) != NO_ERROR)
    {
      close_device ();
      fprintf (stderr, "Cannot open application %s\n", APP_DF);
      exit (-1);
    }

    usb_send (ACTION_DF_INIT, 0, NULL, NULL, NULL);

    if (mc_detect () == MC_OK)
    {
      u8 name[100];

      printf ("Memory card detected\n");
      mc_get_label (name);
      printf ("Card label: %s\n\n", name);
    }
    else if (mc_detect () == MC_NOT_PRESENT)
    {
      printf ("Memory card NOT PRESENT\n");
      close_device ();
      exit (1);
    }
    else if (mc_detect () == MC_NOT_READY)
    {
      printf ("Memory card NOT FORMATED\n\n");
      exit (1);
    }

    f = cd (path);
    if (f < 0)
    {
      fprintf (stderr, "Path does not exist\n");
      exit (1);
    }
    f = create_file (f, name, sort);
    if (f < 0)
    {
      fprintf (stderr, "ERROR %d\n", f);
      exit (1);
    }
    if (ext)
    {
      u8 e[3];

      for (i = 0; i < 3; i++)
	if (i < strlen (ext))
	  e[i] = ext[i];
	else
	  e[i] = 0;
      mc_set_type (f, e);
    }

#define STEP	255

    fd = mc_open (f, FILE_APPEND);
    i = 0;
    while (i < s_size)
    {
      if (s_size - i > STEP)
	len = STEP;
      else
	len = s_size - i;
      mc_append (fd, &s_buf[i], len);
      i = i + len;
      printf ("#");
      fflush (stdout);
    }
    mc_close (fd);

    printf ("\n");
    printf ("RESULT %04X%02X\n", fs_ino_block (f), fs_ino_index (f));
  }
  else
  {
// from card
    open_device (dev);
    if (select_app (APP_DF) != NO_ERROR)
    {
      close_device ();
      fprintf (stderr, "Cannot open application %s\n", APP_DF);
      exit (-1);
    }

    usb_send (ACTION_DF_INIT, 0, NULL, NULL, NULL);

    if (mc_detect () == MC_OK)
    {
      u8 name[100];

      printf ("Memory card detected\n");
      mc_get_label (name);
      printf ("Card label: %s\n\n", name);
    }
    else if (mc_detect () == MC_NOT_PRESENT)
    {
      printf ("Memory card NOT PRESENT\n");
      close_device ();
      exit (1);
    }
    else if (mc_detect () == MC_NOT_READY)
    {
      printf ("Memory card NOT FORMATED\n\n");
      exit (1);
    }

    sscanf (argv[s_src], "%x", &i);
    f = fs_ino_p (i >> 8, i & 0xFF);

    fd = mc_open (f, FILE_READ);
    if (fd == NULL)
    {
      fprintf (stderr, "Cannot open %x\n", f);
      goto X1;
    }
    s_size = get_len (f);
    printf ("Size %d\n", s_size);
    s_buf = malloc (s_size);
    fs = fopen (d_name, "w");
    i = 0;
    while (i < s_size)
    {
      if (s_size - i > STEP)
	len = STEP;
      else
	len = s_size - i;
      mc_read (fd, i, &s_buf[i], len);
      printf ("#");
      fflush (stdout);
      {
	u16 k = 0;

	while (k < len)
	{
	  fputc (s_buf[i + k], fs);
	  k++;
	}
      }
      i = i + len;
    }

    printf ("\n");
    mc_close (fd);
    fclose (fs);
  }

X1:
  close_device ();
}
